//==============================================================================
// ARDUINOPRINT - Adds Arduino print compatible classes to allow code to use
// 		things like Serial.print.
//
//  Original Arduino print code
//  Copyright (c) 2008 David A. Mellis.  All right reserved.
//
// Version 0.1 - Work in Progress
//
// Major Contributers: Kurt Eckhardt 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//==============================================================================
#ifndef _ARDUINO_PRINT_H_
#define _ARDUINO_PRINT_H_


#define PSIZE_DEC 10
#define PSIZE_HEX 16
#define PSIZE_OCT 8
#define PSIZE_BIN 2
#define PSIZE_BYTE 0

class Print;

typedef void (*PWRITECB)(Print*,byte) ;

class Print
{
  private:
    void printNumber(unsigned long, byte);
    void printFloat(double, byte);

	// Write callback function
	PWRITECB _pwritecb;
  public:
    void write(byte);
    void write(const char *str);
    void write(const byte *buffer, byte size);

    void print(const char[]);
    void print(char, int = PSIZE_BYTE);
    void print(unsigned char, int = PSIZE_BYTE);
    void print(int, int = PSIZE_DEC);
    void print(unsigned int, int = PSIZE_DEC);
    void print(long, int = PSIZE_DEC);
    void print(unsigned long, int = PSIZE_DEC);

    void println(const char[]);
    void println(char, int = PSIZE_BYTE);
    void println(unsigned char, int = PSIZE_BYTE);
    void println(int, int = PSIZE_DEC);
    void println(unsigned int, int = PSIZE_DEC);
    void println(long, int = PSIZE_DEC);
    void println(unsigned long, int = PSIZE_DEC);
    void println(void);
    
    // simple function to set the callback...
	void setWriteCB(PWRITECB pwritecb);
};

extern void HardwareSerialCB(Print *p, byte b);

class HardwareSerial : public Print
{
	private:
		byte _bWhich;		//
	friend  void HardwareSerialCB(Print *p, byte b);
	public:
		HardwareSerial(byte bWhich=0);	// Which port 0=HSERIAL 2=HSERIAL2
		void begin(long);
		void end();
		byte available(void);
		int read(void);
		void flush(void);
		void write(byte);
		using Print::write; // pull in write(str) and write(buf, size) from Print
};

extern void SoftwareSerialCB(Print *p, byte b);
class SoftwareSerial : public Print
{
	private:
		byte _bPinOut;	// May define with one or two pins...
		byte _bPinIn;	//
		word _wMode;	// what mode are we outputting...
	friend  void SoftwareSerialCB(Print *p, byte b);
	public:
		SoftwareSerial(byte bPinIn, byte bPinOut);	// Which IO Pins
		void begin(word wMode);
		void end();
		byte available(void);
		int read(unsigned long lTimeout=0);
		void flush(void);
		void write(byte);
	    void write(const byte *buffer, byte size);
		using Print::write; // pull in write(str) and write(buf, size) from Print
};


#endif /* _ARDUINO_PRINT_H_ */
