//=============================================================================
// HServo.c - This file will contain the support functions for us to semi 
// 		emulate the HServo subsystem that is supported in basic.
//==============================================================================
#if 0	// Playing with maybe Arduino format of commands to start off???
#include "BapLib.h"
#ifdef __cplusplus
extern "C" {
#endif

#include "hservo.h"


//-----------------------------------------------------------------------------
// Define data types used in the group servo movement functions
//-----------------------------------------------------------------------------
typedef struct _msrvoitem {
	HSPOS		hspTo;							// this is in our .5uS units
	HSPOS		hspPrev;						// dito
	short		sDeltaDS1;						// Delta per time unit
} MSRVOITEM, *PMSERVOITEM;

typedef struct _msrvodata {
	unsigned  	uWhichServosActive;				// Which servos are active - bit 0 is item 0...
	SERVO_LIST* servos;							// The servos
	uint16_t	servo_cycle;					// Cycle time
//	TICK_COUNT 	tcStart;
//	TICK_COUNT 	tcWait;
	MSRVOITEM	asrvi[MNUMSERVOS];				// array of servo data
} MSERVODATA;

#define MSPERSGMCALLBACK  50					// BUGBUG:: Set to ms will change to US ..hard coded may try to set depending of differences

static MSERVODATA s_msd = {0, 0};
static short s_cSGMCallback = 0;				// debug stuff...

extern void SGMCallback(void* data, TICK_COUNT lasttime, TICK_COUNT overflow);

//#define DEBUG_TIMING
#ifdef DEBUG_TIMING
#define 	TCDEBUG_SIZE 200
unsigned	g_ctcDebug;
TICK_COUNT	g_atcDebug[TCDEBUG_SIZE];

#endif

//==============================================================================
// Init the ServoGroupMove data.
//==============================================================================
void HServoInit(void)			// init our stuff
{
}

//==============================================================================
// This is the main Callback function that is called to update the servo positions
// for the group moves.  This function is a callback function that is called by
// the background scheduler.
//==============================================================================
extern "C" void TIMERAV() __attribute__ ((interrupt_handler));
extern "C" void TIMERAV()
{
	toggle(P6);			
	lTimerCnt += 256;		// increment our counter TCA is the low order byte so increment past it...
	IRR1.BIT.IRRTA = 0;		// clear out the overflow flag
}

extern "C" void TIMERW() __attribute__ ((interrupt_handler));
extern "C" void TIMERW()
{
	// Lets see what we woke up for.
	if (TW.TMSRW.BIT.IMFA)	// GRA Match?
	{
	}
	if (TW.TMSRW.BIT.IMFB)	// GRB Match?
	{
	}
	if (TW.TMSRW.BIT.IMFC)	// GRC Match?
	{
	}
	if (TW.TMSRW.BIT.IMFD)	// GRD Match?
	{
	}

	PMSERVOITEM psrvi;
	short i;
	unsigned uMask;				// mask for which servo number
	unsigned uStillActive;		// Bits for which servos have work still to go
	short shspPrev;

	// first see if we have anything to do.
	if (s_msd.uWhichServosActive)
	{
#ifdef DEBUG_VERBOSE
		rprintfInit(USB_ACTIVATE);
		rprintf("FProcessServoGroupMove - something active\n");
#endif
#ifdef DEBUG_TIMING
		if (g_ctcDebug < TCDEBUG_SIZE)
			g_atcDebug[g_ctcDebug++] = lasttime;
#endif
		uStillActive = 0;				// 
		uMask = 0x1;					// set mask
		for (i=0;i<MNUMSERVOS;i++)
		{
			// now increment each servo
			psrvi = &s_msd.asrvi[i];						// pointer to the data associated with this servo

			// Since this  happens on an interrupt basis, only update thos items that are in the mask of items
			// this will keep us from having our interrupt start processing a group move, until the complete
			// update has been completed.
			if ((s_msd.uWhichServosActive & uMask) && psrvi->cCyclesLeft)
			{
				SERVO* servo = (SERVO *)pgm_read_word(&s_msd.servos[i]);

				// Ok this one has something to do
				if (--(psrvi->cCyclesLeft))
				{		
					uStillActive |= uMask;					// Set the bit for this IO pin to say it is still active...
					shspPrev = psrvi->sDS1;					// save the current drive speed away
				 	psrvi->sDS1 += psrvi->sDeltaDS1; 		// increment the  delta per cycle in 10ths of unit
					if ((shspPrev/10) != (psrvi->sDS1/10))
						act_setSpeed(servo,  psrvi->sDS1/10);
				}
				else
					act_setSpeed(servo,  psrvi->hspTo);		// Last time through the loop make sure we end up at the right final space. 
			}
			uMask <<= 1;					// go up to the next bit.
		}	
	
		s_msd.uWhichServosActive = uStillActive;		// remember if there is still stuff left to do
	}

	// If there is still group moves going on, we will rescheule ourself.
	s_cSGMCallback++;
	if (s_msd.uWhichServosActive)
	{
		scheduleJob(&SGMCallback, 0, lasttime, (TICK_COUNT)(MSPERSGMCALLBACK)*1000);	// reschedule our self, rember to conver mS to uS.
	}
}

//==============================================================================
//[ServoGroupMove] - This is the main servo interface.  This function takes new positions for each of the Legs servos
//             in 10s of a degree as well as the time we want the movement to take.  We then compute the speed
//             of each of the servos, such that all of the servos complete their moves at the same time.
//			  This function does not return until the servos have completed their movement.
//
//==============================================================================

//
// We will setup the servo moves to evenutally work off a timer. For now may debug with it working off of the main loop, with it
// checking to see if the elapsed time has happened before processing. 

void ServoGroupMove (short iServoFirst, short cServos, HSPOS *ads, short sMoveTime)
{
	short i;
	short cCycles;
	PMSERVOITEM psrvi;
	unsigned uNewTimedServos;
	unsigned uWhichServosActiveBefore;
#ifdef DEBUG
	if (g_fDebugOutput)
	{
		rprintfInit(USB_ACTIVATE);
		rprintf("ServoGroupMove: %d %d %d %d:", iServoFirst, cServos, sMoveTime, s_cSGMCallback);
		s_cSGMCallback = 0;
		for (i=0; i < cServos; i++)
			rprintf("  %d", ads[i]);
		rprintf("\n");
	}
#endif
	if (sMoveTime)
		cCycles = (sMoveTime+MSPERSGMCALLBACK-1) / MSPERSGMCALLBACK; 	// try to figure how many cycles this is BUGBUG:: we know all of our times are currently n*50
	else
		cCycles = 0;

	uNewTimedServos = 0;
	for (i=0;i<cServos;i++)
	{	
		psrvi = &(s_msd.asrvi[iServoFirst+i]);		// pointer to the data associated with this servo

		// We will do some of this manipulation within a critical section as to keep the
		// data consistent.  Note: there is an if in this so make sure both paths are handled.
		CRITICAL_SECTION_START;
		psrvi->hspPrev = psrvi->hspTo;				// Save away the previous HSPOS
		psrvi->hspTo = ads[i];						// save away the new DS
		if ((psrvi->hspPrev != psrvi->hspTo) && cCycles)
		{
			psrvi->sDeltaDS1 = ((psrvi->hspTo - psrvi->hspPrev) * 10) / cCycles;
			psrvi->cCyclesLeft = cCycles;			
			psrvi->sDS1 = psrvi->hspPrev * 10;		// previous * 10 (.1 units)
			CRITICAL_SECTION_END;
			uNewTimedServos |= (1 << (iServoFirst+i));	// set the appropriat bit to true
#ifdef DEBUG_VERBOSE
			if (g_fDebugOutput)
				rprintf(" %d/%d/%d", iServoFirst+i, (short)psrvi->hspTo, (short)psrvi->sDeltaDS1);
#endif		
		}
		else
		{
			psrvi->cCyclesLeft = 0;
			CRITICAL_SECTION_END;
#ifdef DEBUG_VERBOSE
			if (g_fDebugOutput)
				rprintf(" %d/IM/%d", iServoFirst+i, psrvi->hspTo);
#endif
			SERVO* servo = (SERVO *)pgm_read_word(&s_msd.servos[iServoFirst+i]);
			act_setSpeed(servo,  psrvi->hspTo);			// if same or count 0 just set it now... May noop at servo.
		}
	}

	CRITICAL_SECTION_START;
	// Update the list of which servos have group move data.  Do this under critical section to make sure it keeps consistent. 
	// remember the state of which servos were active before we or it in, such that we know if our main callback function is running or not.
	uWhichServosActiveBefore = s_msd.uWhichServosActive;
	s_msd.uWhichServosActive |= uNewTimedServos;
	CRITICAL_SECTION_END;

#ifdef DEBUG_VERBOSE
	// setup to output back to our USB port
	if (g_fDebugOutput)
		rprintf("\n");
#endif

	if ((uWhichServosActiveBefore == 0) &&  s_msd.uWhichServosActive)
	{
		scheduleJob(&SGMCallback, 0, clockGetus(), ((TICK_COUNT)MSPERSGMCALLBACK)*1000);	// Schedule up our group move - start at the next MS
	}
#ifdef DEBUG
	else if (g_fDebugOutput)
	{
		rprintf("SGM: NCB! %x %x %d\n", uWhichServosActiveBefore, s_msd.uWhichServosActive, cCycles);
		s_cSGMCallback = 0;
		for (i=0; i < cServos; i++)
			rprintf("  %d", ads[i]);
		rprintf("\n");
	}
#endif


	// Now return the main loop will do the calling out to the processServoGroupMove to monitor and update
	// the actual moves.

}

//==============================================================================
//[WhichGroupMoveServosActive] - This function does the main processing to move each
//			of the servos during a group move. 
//	 Return: Boolean - Are we still active or are all of the servos at their
//			end points?
//==============================================================================
unsigned WhichGroupMoveServosActive(void)
{
	// Could put inline...
	return s_msd.uWhichServosActive;
}

//==============================================================================
//[WaitForGMToComplete] - This function will spin until the specified
//			servos are no longer active in a group move.
//
//	Inputs: <Servo Mask> - a bit for each servo that we wish to wait for...
//==============================================================================
void WaitForGMToComplete(unsigned uSGMMask)
{
	// Could put inline...
	while ((s_msd.uWhichServosActive & uSGMMask) != 0)
	{
		clockWaitms(1);
	}
}


//==============================================================================
//[DumpGPTimings] - This is a debug function that I used to help find out 
//	 why some moves were not taking as long as expected.
//==============================================================================
void DumpGPTimings(void)
{
#ifdef DEBUG_TIMING
	unsigned i;
	TICK_COUNT tc;

	rprintfInit(USB_ACTIVATE);
	rprintf("\nDump Servo GP Timings\n");

	for (i= 0; i < g_ctcDebug-1; i++)
	{
		tc = g_atcDebug[i+1]-g_atcDebug[i];

		if (tc > (TICK_COUNT)0xffff)
			rprintf(" XXXX");
		else
			rprintf("  %u", (short)tc);
		if ((i & 0x7) == 0x7)
			rprintf("\n");
	}

	rprintf("\n");
	g_ctcDebug = 0;
#endif
}

#ifdef __cplusplus
}
#endif
#endif //0
