//==============================================================================
// IOPINS - Handle some of the IO functions of the BAP
//
// Version 0.1 - Work in Progress
//
// Major Contributers: Kurt Eckhardt 
// 		Robert V Mahoney Jr
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//==============================================================================
#include "BapLib.h"
#include <stdarg.h>

#ifdef __cplusplus
extern "C" {
#endif

#define _LOW_ 						0
#define _HIGH_						1

#define _BA_PIN_MODE_IO_			0
#define _BA_PIN_MODE_FUNCTION_		1

#define _BA_PIN_MODE_INPUT_			0
#define _BA_PIN_MODE_OUTPUT_		1

#define _BA_ADC_MODE_SINGLE_		0
#define _BA_ADC_MODE_SCAN_			1

#define _BA_ADC_CLOCK_134_			0
#define _BA_ADC_CLOCK_70_			1

enum _PCR_SHADOW_REGS_ {_PCR1_SHADOW_, _PCR2_SHADOW_, _PCR3_SHADOW_, _PCR4_SHADOW_, _PCR5_SHADOW, _PCR6_SHADOW_, _PCR7_SHADOW_, _PCR8_SHADOW_, _PCR_SHADOW_COUNT_};
u8 bap_bpcrshadowregs [_PCR_SHADOW_COUNT_];


#ifdef H8_FAMILY_3694	
const PORTITEM	g_PortTable28[] =
{
	{&IO.PDR5.BYTE, 0x1, (PUR_5 | PMR_5)},	// P0
	{&IO.PDR5.BYTE, 0x2, (PUR_5 | PMR_5)},	// P1
	{&IO.PDR5.BYTE, 0x4, (PUR_5 | PMR_5)},	// P2
	{&IO.PDR5.BYTE, 0x8, (PUR_5 | PMR_5)},	// P3
	{&IO.PDR5.BYTE, 0x10, (PUR_5 | PMR_5)},	// P4
	{&IO.PDR5.BYTE, 0x20, (PUR_5 | PMR_5)},	// P5
	{&IO.PDR5.BYTE, 0x40, 0},	// P6
	{&IO.PDR5.BYTE, 0x80, 0},	// P7
	{&IO.PDR8.BYTE, 0x1, 0},	// P8 - Does support pull-up but on P15?  so for now say no
	{&IO.PDR8.BYTE, 0x2, 0},	// P9
	{&IO.PDR8.BYTE, 0x4, 0},	// P10
	{&IO.PDR8.BYTE, 0x8, 0},	// P11
	{&IO.PDR8.BYTE, 0x10, 0},	// P12 - Does support pull-up on P10; so far now say no
	{&IO.PDR8.BYTE, 0x20, 0},	// P13
	{&IO.PDR8.BYTE, 0x40, 0},	// P14
	{&IO.PDR8.BYTE, 0x80, 0},	// P15
	{&IO.PDR1.BYTE, 0x2, (PUR_1)},	// P16
	{&IO.PDR1.BYTE, 0x4, (PUR_1)},	// P17
	{&IO.PDR1.BYTE, 0x80, (PUR_1 | PMR_1)},	// P18
	{&IO.PDR1.BYTE, 0x40, (PUR_1 | PMR_1)},	// P19
	{0, 0, 0},	//20
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},//30
	{0, 0, 0},
	{&IO.PDR1.BYTE, 0x10, 0},	// S_IN
	{&IO.PDR1.BYTE, 0x10, 0}	// S_OUT
};
#endif

// OK lets define Bap40 pins as well
#ifdef H8_FAMILY_3687
const PORTITEM	g_PortTable40[] =
{
	{&IO.PDR5.BYTE, 0x1, (PUR_5 | PMR_5)},	// P0
	{&IO.PDR5.BYTE, 0x2, (PUR_5 | PMR_5)},	// P1
	{&IO.PDR5.BYTE, 0x4, (PUR_5 | PMR_5)},	// P2
	{&IO.PDR5.BYTE, 0x8, (PUR_5 | PMR_5)},	// P3
	{&IO.PDR5.BYTE, 0x10, (PUR_5 | PMR_5)},	// P4
	{&IO.PDR5.BYTE, 0x20, (PUR_5 | PMR_5)},	// P5
	{&IO.PDR5.BYTE, 0x40, 0},	// P6
	{&IO.PDR5.BYTE, 0x80, 0},	// P7
	{&IO.PDR6.BYTE, 0x1, 0},	// P8
	{&IO.PDR6.BYTE, 0x2, 0},	// P9
	{&IO.PDR6.BYTE, 0x4, 0},	// P10
	{&IO.PDR6.BYTE, 0x8, 0},	// P11
	{&IO.PDR6.BYTE, 0x10, 0},	// P12
	{&IO.PDR6.BYTE, 0x20, 0},	// P13
	{&IO.PDR6.BYTE, 0x40, 0},	// P14
	{&IO.PDR6.BYTE, 0x80, 0},	// P15
	{&IO.PDR3.BYTE, 0x1,  0},	// P16
	{&IO.PDR3.BYTE, 0x2, 0},	// P17
	{&IO.PDR3.BYTE, 0x4, 0},	// P18
	{&IO.PDR3.BYTE, 0x8,  0},	// P19
	{&IO.PDR3.BYTE, 0x10, 0},	// P20
	{&IO.PDR3.BYTE, 0x20, 0},	// P21
	{&IO.PDR3.BYTE, 0x40, 0},	// P22
	{&IO.PDR3.BYTE, 0x80, 0},	// P23
	{&IO.PDR1.BYTE, 0x1, (PUR_1 | PMR_1)},	// P24
	{&IO.PDR1.BYTE, 0x2, (PUR_1 )},		// P25 - PMR does not match
	{&IO.PDR1.BYTE, 0x4, (PUR_1 | PMR_1)},	// P26
	{&IO.PDR2.BYTE, 0x8, 0},	// P27
	{&IO.PDR1.BYTE, 0x10, (PUR_1 | PMR_1)},	// P28
	{&IO.PDR1.BYTE, 0x20, (PUR_1 | PMR_1)},	// P29
	{&IO.PDR1.BYTE, 0x40, (PUR_1 | PMR_1)},	// P30
	{&IO.PDR1.BYTE, 0x80, (PUR_1 | PMR_1)},	// P31
	{&IO.PDR2.BYTE, 0x2, 0},	// S_IN (P21)
	{&IO.PDR2.BYTE, 0x4, 0}	// S_OUT TXD(P22)
};


// OK lets define Arc32 pins as well
const PORTITEM	g_PortTableArc32[] =
{
	{&IO.PDR5.BYTE, 0x10, (PUR_5 | PMR_5)},		// P0		servo0		(P54/WKP4)
	{&IO.PDR5.BYTE, 0x20, (PUR_5 | PMR_5)},		// P1 		servo1		(P55/WKP5/ADTRG)
	{&IO.PDR1.BYTE, 0x1, (PUR_1 | PMR_1)},		// P2 		servo2		(P10/TMOW)
	{&IO.PDR1.BYTE, 0x2, (PUR_1 )},				// P3 		servo3		(P11/PWM)
	{&IO.PDR1.BYTE, 0x4, (PUR_1 | PMR_1)},		// P4 		servo4		(P12)
	{&IO.PDR7.BYTE, 0x20, 0},					// P5 		servo5		(P75/TMCIV)
	{&IO.PDR7.BYTE, 0x40, 0},					// P6 		servo6		(P76/TMOV)
	{&IO.PDR2.BYTE, 0x8, 0},					// P7 		servo7		(P23)
	{0,0,0},									// P8      			
	{0,0,0},									// P9      			
	{0,0,0},									// P10      			
	{0,0,0},									// P11      			
	{0,0,0},									// P12      			
	{0,0,0},									// P13      			
	{0,0,0},									// P14      			
	{0,0,0},									// P15      			
	{&IO.PDR5.BYTE, 0x8, (PUR_5 | PMR_5)},		// P16 	servo16			(P53/WKP3)
	{&IO.PDR5.BYTE, 0x4, (PUR_5 | PMR_5)},		// P17 	servo17			(P52/WKP2)
	{&IO.PDR5.BYTE, 0x2, (PUR_5 | PMR_5)},		// P18 	servo18			(P51/WKP1)
	{&IO.PDR5.BYTE, 0x1, (PUR_5 | PMR_5)},		// P19 	servo19			(P50/WKP0)
	{&IO.PDR3.BYTE, 0x1, 0},					// P20 	servo20			(P30)
	{&IO.PDR3.BYTE, 0x2, 0},					// P21 	servo21			(P31)
	{&IO.PDR3.BYTE, 0x4, 0},					// P22 	servo22			(P32)
	{&IO.PDR3.BYTE, 0x8, 0},					// P23 	servo23			(P33)
	{0,0,0},									// P24
	{0,0,0},									// P25      			
	{0,0,0},									// P26      			
	{0,0,0},									// P27      			
	{&IO.PDR1.BYTE, 0x80, (PUR_1 | PMR_1)},		// P28 	servo28			(P17/IRQ3/TRGV)
	{&IO.PDR1.BYTE, 0x40, (PUR_1 | PMR_1)},		// P29 	servo29			(P16/IRQ2)
	{&IO.PDR1.BYTE, 0x20, (PUR_1 | PMR_1)},		// P30 	servo30			(P15/IRQ1/TMIB1)
	{&IO.PDR1.BYTE, 0x10, (PUR_1 | PMR_1)},		// P31	servo41			(P14/IRQ0)
	{0,0,0},									// P32      			
	{0,0,0},									// P33      			
	{&IO.PDR5.BYTE, 0x80, (PUR_5 | PMR_5)},		// P34		AUX1 pin1		(P57/SCL)
	{&IO.PDR5.BYTE, 0x40, (PUR_5 | PMR_5)},		// P35		AUX1 pin2		(P56/SDA)
	{&IO.PDR7.BYTE, 0x4, 0},					// P36		AUX1 pin3		(P72/TXD_2)
	{&IO.PDR7.BYTE, 0x1, 0},					// P37		AUX1 pin4		(P70/SCK3_2)
	{&IO.PDR7.BYTE, 0x2, 0},					// P38		AUX1 pin5		(P71/RXD_2)
	{&IO.PDR3.BYTE, 0x80, 0},					// P39		AUX1 pin6		(P37) and (PB3)
	{&IO.PDR2.BYTE, 0x10, 0},					// P40		AUX2 pin2		(P24) with pullup
	{&IO.PDR8.BYTE, 0x20, 0},					// P41		AUX2 pin4		(P85) with pullup
	{&IO.PDR8.BYTE, 0x40, 0},					// P42		AUX2 pin6		(P86) with pullup
	{&IO.PDR8.BYTE, 0x80, 0},					// P43		AUX2 pin8		(P87) with pullup
	{&IO.PDR3.BYTE, 0x40, 0},					// P44		Status LED		(P36)
	{&IO.PDR7.BYTE, 0x10, 0},					// P45		Batt LED		(P74/TMRIV)
	{&IO.PDR3.BYTE, 0x10, 0},					// P46		MUX A			(P34)
	{&IO.PDR3.BYTE, 0x20, 0},					// P47		MUX B			(P35)
	{&IO.PDR6.BYTE, 0x1, 0},					// P48		U24 X 		(P60/FTIOA0)
	{&IO.PDR6.BYTE, 0x2, 0},					// P49		U24 Y			(P61/FTIOB0)
	{&IO.PDR6.BYTE, 0x4, 0},					// P50		U21 X			(P62/FTIOC0)
	{&IO.PDR6.BYTE, 0x8, 0},					// P51		U21 Y			(P63/FTIOD0)
	{&IO.PDR6.BYTE, 0x10, 0},					// P52		U22 X			(P64/FTIOA1)
	{&IO.PDR6.BYTE, 0x20, 0},					// P53		U22 Y			(P65/FTIOB1)
	{&IO.PDR6.BYTE, 0x40, 0},					// P54		U23 Y			(P66/FTIOC1)
	{&IO.PDR6.BYTE, 0x80, 0},					// P55		U23 X			(P67/FTIOD1)
	{&IO.PDRB.BYTE, 0x1, 0},					// P56		VS1			(PB0/AN0)
	{&IO.PDRB.BYTE, 0x2, 0},					// P57		VL			(PB1/AN1)
	{&IO.PDRB.BYTE, 0x4, 0},					// P58		VS2			(PB2/AN2)
	{&IO.PDR2.BYTE, 0x2, 0},					// S_IN (P21)
	{&IO.PDR2.BYTE, 0x4, 0}						// S_OUT TXD(P22)
};	
#endif

//Pro 28 H8/3664F Pro 28 H8/3664F
//P0 P50/WKP0 PB0/AN0 P8 P80/FTCI P15/IRQ1
//P1 P51/WKP1 PB1/AN1 P9 P81/FTIOA P74/TMRIV
//P2 P52/WKP2 PB2/AN2 P10 P82/FTIOB
//P3 P53/WKP3 PB3/AN3 P11 P83/FTIOC
//P4 P54/WKP4 P12 P84/FTIOD P10/TMOW
//P5 P55/WKP5/ADTRG P13 P85 P20/SCK3
//P6 P56/SDA P14 P86 P21/RXD
//P7 P57/SCL P15 P87 P22/TXD
//P16 P11 PB7/AN7 P18 P16/IRQ2 PB5/AN5
//P17 P12 PB6/AN6 P19 P17/IRQ3 PB4/AN4


//-----------------------------------------------------------------------------
// Globals that may be used by functions and callers
//-----------------------------------------------------------------------------
PORTITEM	(*g_pPortTable)[];		// Pointer to the current port table
byte 		g_cPortItems;			// How many items are in the current Port table
u8			g_bWhichBap;			// may need to know at run time if running on BAP40 vs Arc32...



//=============================================================================
// bap_initialize
//=============================================================================
void _bap_initialize (u8 bWhichBap)
{
	u8 i;

	// save away the run time value for which bap we are running on.
	g_bWhichBap = bWhichBap;

	// disable IRQs
	IO.PMR1.BIT.IRQ0 = _BA_PIN_MODE_IO_;
	IO.PMR1.BIT.IRQ1 = _BA_PIN_MODE_IO_;
	IO.PMR1.BIT.IRQ2 = _BA_PIN_MODE_IO_;
	IO.PMR1.BIT.IRQ3 = _BA_PIN_MODE_IO_;

	// disable interrupts
	IENR1.BIT.IEN0 = _BA_PIN_MODE_IO_;
	IENR1.BIT.IEN1 = _BA_PIN_MODE_IO_;
	IENR1.BIT.IEN2 = _BA_PIN_MODE_IO_;
	IENR1.BIT.IEN3 = _BA_PIN_MODE_IO_;

	// set PORT5 wakeup pins as IO
	IO.PMR5.BIT.WKP0 = _BA_PIN_MODE_IO_;
	IO.PMR5.BIT.WKP1 = _BA_PIN_MODE_IO_;
	IO.PMR5.BIT.WKP2 = _BA_PIN_MODE_IO_;
	IO.PMR5.BIT.WKP3 = _BA_PIN_MODE_IO_;
	IO.PMR5.BIT.WKP4 = _BA_PIN_MODE_IO_;
	IO.PMR5.BIT.WKP5 = _BA_PIN_MODE_IO_;
	// set TMOW as IO pin
	IO.PMR1.BIT.TMOW = _BA_PIN_MODE_IO_;
	// set TXD as IO pin
	IO.PMR1.BIT.TXD = _BA_PIN_MODE_IO_;
	// set RXD as IO pin
	SCI3.SCR3.BIT.RE = _BA_PIN_MODE_IO_;
	SCI3.SSR.BYTE = 0;			// clear out any status

	// set SCK3 as IO pin
	SCI3.SCR3.BIT.CKE = _BA_PIN_MODE_IO_;
	SCI3.SMR.BIT.COM = _BA_PIN_MODE_IO_;

	SCI3.SCR3.BIT.RIE = _FALSE_;		// disable interrupts when we receive a character or error
	SCI3.SCR3.BIT.TIE = _FALSE_;		// disable interrupts when ready for next byte to transmit
	SCI3.SCR3.BIT.TEIE = _FALSE_;		// For now don't need to enable interrupt when transmit complete...


	IIC2.ICIER.BYTE = 0;				// Make sure I2C is not going to generate any interrupts.
	IIC2.ICCR1.BIT.ICE = 0;				// Likewise it is disabled...

	// set PORT pins as input - handle all of them here.
	for (i=0; i< _PCR_SHADOW_COUNT_; i++)
		bap_bpcrshadowregs [i] = 0;

	// set PORT pins from shadow registers - All of them that are common...
	IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
	IO.PCR2 = bap_bpcrshadowregs [_PCR2_SHADOW_];
	IO.PCR5 = bap_bpcrshadowregs [_PCR5_SHADOW];
	IO.PCR7 = bap_bpcrshadowregs [_PCR7_SHADOW_];
	IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
	// disable input pullups
	IO.PUCR1.BYTE = 0x00;
	IO.PUCR5.BYTE = 0x00;

	// set RXD as IO pin
	SCI3.SCR3.BIT.RE = _BA_PIN_MODE_IO_;

#ifdef H8_FAMILY_3694
	// Currently we have only define BAP28
	g_pPortTable = (PORTITEM (*)[])&g_PortTable28;	// point to the count of 
	g_cPortItems = sizeof(g_PortTable28)/sizeof(PORTITEM);
	
	// disable FTIO
	// Make sure TimerW interrupts start off disabled
	TW.TIERW.BYTE = 0;		// All interrupts disabled
	TW.TIOR0.BIT.IOA = 0x00;
	TW.TIOR0.BIT.IOB = 0x00;
	TW.TIOR1.BIT.IOC = 0x00;
	TW.TIOR1.BIT.IOD = 0x00;
	// disable analog to digital
	AD.ADCSR.BIT.ADST = 0;
	AD.ADCSR.BYTE = 0;		// try to clear out everything of AtoD
	// disable timer interrupts
	IENR1.BIT.IENTA = 0; // timera
	// disable timer counting
	TW.TMRW.BIT.CTS = 0; // timerw
#else
	if (g_bWhichBap == H8_3687_BAP40) {
		g_pPortTable = (PORTITEM (*)[])&g_PortTable40;	// point to the count of 
		g_cPortItems = sizeof(g_PortTable40)/sizeof(PORTITEM);
	} else {
		g_pPortTable = (PORTITEM (*)[])&g_PortTableArc32;	// point to the count of 
		g_cPortItems = sizeof(g_PortTableArc32)/sizeof(PORTITEM);
	}	

	// Make sure the other shadow registers for 3687 are set
	IO.PCR3 = bap_bpcrshadowregs [_PCR3_SHADOW_];
	IO.PCR6 = bap_bpcrshadowregs [_PCR6_SHADOW_];


	// set TXD2 as IO pin
	IO.PMR1.BIT.TXD2 = _BA_PIN_MODE_IO_;

	// For now both same for Bap40 and Arc32...
	// set SCK3_2 as IO pin
	SCI3_2.SCR3.BIT.CKE = _BA_PIN_MODE_IO_;
	SCI3_2.SMR.BIT.COM = _BA_PIN_MODE_IO_;

	SCI3_2.SCR3.BIT.RIE = _FALSE_;		// disable interrupts when we receive a character or error
	SCI3_2.SCR3.BIT.TIE = _FALSE_;		// disable interrupts when ready for next byte to transmit
	SCI3_2.SCR3.BIT.TEIE = _FALSE_;		// For now don't need to enable interrupt when transmit complete...
	// set RXD as IO pin
	SCI3_2.SCR3.BIT.RE = _BA_PIN_MODE_IO_;

	// Make sure TimerZ interrupts start off disabled
	TZ0.TIER.BYTE = 0;		// All interrupts disabled
	TZ1.TIER.BYTE = 0;		// All interrupts disabled
	
	TZ0.TIORA.BIT.IOA = 0x00;
	TZ0.TIORA.BIT.IOB = 0x00;
	TZ0.TIORC.BIT.IOC = 0x00;
	TZ0.TIORC.BIT.IOD = 0x00;

	TZ1.TIORA.BIT.IOA = 0x00;
	TZ1.TIORA.BIT.IOB = 0x00;
	TZ1.TIORC.BIT.IOC = 0x00;
	TZ1.TIORC.BIT.IOD = 0x00;

	TZ.TSTR.BYTE=0;		// start of with both counters not running
	TZ.TOER.BYTE=0xff;	// I believe that sets all to normal pin...
	TZ.TFCR.BYTE=0;		// 

	// disable analog to digital
	AD.ADCSR.BIT.ADST = 0;
	AD.ADCSR.BYTE = 0;		// try to clear out everything of AtoD

	// disable timer interrupts
	IENR1.BIT.IENTA = 0; 	// timer - hung...
	IENR2.BIT.IENTB1 = 0;	// Timer b1 starts off not interrupting

	
#endif

}



// pause 0.5 microseconds
//=============================================================================
// pauseus (u32 lpause)
//=============================================================================
void pauseus (u32 _lpause)
{
	u32 lpause;
#ifdef H8_FAMILY_3687
	_lpause = (_lpause * 5)/2;		// need to take 20mhz instead of 16 into account
#else
	_lpause *= 2;					// Loop takes 8 clocks per which is .5us so *2
#endif	

	for (lpause=0; lpause<_lpause; lpause++);
}

//=============================================================================
// pause (u32 lpause) - IN MILLISECONDS
//=============================================================================
void pause (u32 _lpause)
{
	pauseus ((_lpause*1000));
}



//=============================================================================
// input (u8 _bpin)
//=============================================================================
void input (u8 _bpin, bool fPullup)
{
	// Lets use our table to do the work for us...
	if (_bpin < g_cPortItems)	{
		// Ok lets get the port table item...
		PORTITEM pi = (*g_pPortTable)[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR){
			// See if there is a PMR register associated with this pin.
			if ((pi.bPU_PM) & 0xf) {
				volatile register unsigned char *pPMR = &IO.PMR1.BYTE + (pi.bPU_PM & 0xf) - 1;	// get the right PMR register
				*pPMR &= ~(pi.bMask);
			}
			
			// Next lets update PDR/PCR...
			u8 bIOPortIndex = (pi.pPDR - &IO.PDR1.BYTE);
			volatile unsigned char *pPCR = &IO.PCR1 + bIOPortIndex;
			bap_bpcrshadowregs[bIOPortIndex] &= ~(pi.bMask);				// Update shadow mask to say that the pin is now an input pin
			*pPCR = bap_bpcrshadowregs[bIOPortIndex];
			
			// Now lets see if we have a PUCR register (pull-up) to update for this pin
			if ((pi.bPU_PM) >> 4) {
				volatile register unsigned char *pPUCR = &IO.PUCR1.BYTE + (pi.bPU_PM >> 4) - 1;	// get the right PUCR register
				if (fPullup)
					*pPUCR |= pi.bMask;
				else
					*pPUCR &= ~(pi.bMask);
			}
			
		}
	}	
			
}

//=============================================================================
// output(u8 _bpin)
//=============================================================================
void output (u8 _bpin)
{
	// Lets use our table to do the work for us...
	if (_bpin < g_cPortItems) {
		// Ok lets get the port table item...
		PORTITEM pi = (*g_pPortTable)[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR) {
			// See if there is a PMR register associated with this pin.
			if ((pi.bPU_PM) & 0xf) {
				volatile register unsigned char *pPMR = &IO.PMR1.BYTE + (pi.bPU_PM & 0xf) - 1;	// get the right PMR register
				*pPMR &= ~(pi.bMask);
			}
			
			// Next lets update PDR/PCR...
			u8 bIOPortIndex = (pi.pPDR - &IO.PDR1.BYTE);
			volatile unsigned char *pPCR = &IO.PCR1 + bIOPortIndex;
			bap_bpcrshadowregs[bIOPortIndex] |= pi.bMask;				// Ok update shadow mask to say it is output
			*pPCR = bap_bpcrshadowregs[bIOPortIndex];
		}
	}	
			

}

//=============================================================================
// pinMode (u8 _bpin, u8 _bmode);	// some compatibility with Arduino command set.
//=============================================================================
void pinMode (u8 _bpin, u8 _bmode)	// some compatibility with Arduino command set.
{
	if (_bmode == INPUT)
		input(_bpin);
	if (_bmode == OUTPUT)
		output(_bpin);
}

//=============================================================================
// setpin - Set the pin to high or low
//=============================================================================
void setpin (u8 _bpin, u8 _bstate)
{
	if (_bstate == _LOW_)
		low (_bpin);
	else
		high (_bpin);
}

//=============================================================================
// low (u8 _bpin)
//=============================================================================
void low (u8 _bpin)
{
	// Lets use our table to do the work for us...
	if (_bpin < g_cPortItems) {
		// Ok lets get the port table item...
		PORTITEM pi = (*g_pPortTable)[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR)
			PI_LOW(pi);
	}
}

//=============================================================================
// high (u8 _bpin)
//=============================================================================
void high (u8 _bpin)
{
	// Lets use our table to do the work for us...
	if (_bpin < g_cPortItems) {
		// Ok lets get the port table item...
		PORTITEM pi = (*g_pPortTable)[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR)
			PI_HIGH(pi);
	}
}

// set the pin high
//=============================================================================
// Toggle (u8 _bpin)
//=============================================================================
void toggle (u8 _bpin)
{
	// Lets use our table to do the work for us...
	if (_bpin < g_cPortItems) {
		// Ok lets get the port table item...
		PORTITEM pi = (*g_pPortTable)[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR)
			PI_TOGGLE(pi);
	}
}


//=============================================================================
// u8 in (u8 _bpin)
//=============================================================================
u8 in (u8 _bpin)
{
	// Lets use our table to do the work for us...
	if (_bpin < g_cPortItems) {
		// Ok lets get the port table item...
		PORTITEM pi = (*g_pPortTable)[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR)
			return PI_ISHIGH(pi);
	}
	return 0;	
}

//=============================================================================
// u16 adin(u8 _bpin)
//=============================================================================
// analog-to-digital conversion
u16 adin (u8 _bpin)
{
	u16 wadc = 0;

	AD.ADCSR.BIT.ADST = 0; // stop conversion
	AD.ADCSR.BIT.ADIE = _FALSE_; // disable interrupt
	AD.ADCSR.BIT.SCAN = _BA_ADC_MODE_SINGLE_;
	AD.ADCSR.BIT.CKS = _BA_ADC_CLOCK_134_;
	AD.ADCR.BIT.TRGE = _FALSE_; // no external trigger
	switch (_bpin) 	{
		case 0:
		case 1:
		case 2:
		case 3:
			AD.ADCSR.BIT.CH = _bpin;
			break;
		case 16:
			AD.ADCSR.BIT.CH = 7;
			break;
		case 17:
			AD.ADCSR.BIT.CH = 6;
			break;
		case 18:
			AD.ADCSR.BIT.CH = 5;
			break;
		case 19:
			AD.ADCSR.BIT.CH = 4;
			break;
	}
	AD.ADCSR.BIT.ADST = 1; // start conversion
	while (AD.ADCSR.BIT.ADF != 1);
	while (AD.ADCSR.BIT.ADST == 1);
	switch (_bpin) {
		case 0:
		case 19:
			wadc = AD.ADDRA;
			break;
		case 1:
		case 18:
			wadc = AD.ADDRB;
			break;
		case 2:
		case 17:
			wadc = AD.ADDRC;
			break;
		case 3:
		case 16:
			wadc = AD.ADDRD;
			break;
	}
	return (wadc>>6);
}

//=============================================================================
// MultiAdin(u8 badcsr, u16 *pawatds) - Do multiple AtoD conversions. - Not
// 		very abstraced from the hardware, but... pass in the flags to use
//		to start up the AtoDs and then cycle.  We will have at least three
//		cases:
//			1) Startup the scan... specify which channels to use
//			2) Do next set of samples... 
//			3) Shut it down.
//		There may be other cases where we wish to change channels/config, but
//		for now punt (close/restart...)
//=============================================================================
// analog-to-digital conversion
void MultiAdin (u8 badcsr, u16 *pawatds)
{

	if (!pawatds) {
		// Start with easy case.  if pawatds = NULL then stop conversion.
		AD.ADCSR.BYTE = 0; 	// stop the conversion clear out the other fields while I am at it.
									// that way the IO pins are freed up as well...
									
	} else {
		if (!AD.ADCSR.BIT.ADST)	{
			// Was not already started, so lets initialize things...
			AD.ADCSR.BIT.ADF = 0;	       				// Clear the finished a pass flag...
            AD.ADCSR.BIT.ADIE = 0;						// We do not want interrupts
            AD.ADCSR.BIT.SCAN = 1;						// Yes lets start a scan.
            AD.ADCSR.BIT.CKS = (badcsr & 0x8)? 1 : 0;	// set clock speed
            AD.ADCSR.BIT.CH =  badcsr &0x7; 			// Setup which analog channels we want to start

			// now startup the scan                    
            AD.ADCSR.BIT.ADST = 1;       
		}
		
		// Now lets wait for a pass to complete...
		while (AD.ADCSR.BIT.ADF == 0) ;
		
		// Now lets return the Analog conversion data from the registers.
		*pawatds++ = AD.ADDRA >> 6;
		if  (badcsr & 0x3) {
			*pawatds++ = AD.ADDRB >> 6;
			if  (badcsr & 0x2) {
				*pawatds++ = AD.ADDRC >> 6;
				if  (badcsr & 0x1)
					*pawatds++ = AD.ADDRD >> 6;
			}
		}
		AD.ADCSR.BIT.ADF = 0;	       				// so we know if another pass has completed or not
	}
}


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//==============================================================================
// [Preliminary sound stuff] - move to approriate place...
//==============================================================================
// BUGBUG:: move to somewhere...
void Sound (u8 bPin, unsigned long wDur, unsigned long wFreq)			// dur in mS and Freq in Herz.  or should it be like basic???
{
	PORTITEM	piPin;									// The port item for the specified pin
	unsigned long	cCycles;								// the number of cycles to make this sound
	register unsigned long cPerHalfCycle;				// how many times to loop through per half cycle
	register unsigned long i;							// counter to use for the loops
	piPin = (*g_pPortTable)[bPin];							// get the port table item for the specified pin...
	PI_LOW(piPin);
	output(bPin);										// set the pin to output.
	
	// first calculate how many cycles of the tone we will produce
	cPerHalfCycle = (unsigned long)F_CPU/(unsigned long)(wFreq * 2 *8)-3;			// *2(halves) *8(2dec, 2mov, 4bne) guess per count  - guess for setup and high/low
	cCycles = ((unsigned long)wDur * wFreq)/1000;

	while (cCycles--) {
		i = cPerHalfCycle;
		while (--i)	;
		PI_HIGH(piPin);
		i = cPerHalfCycle;
		while (--i)	;
		PI_LOW(piPin);										// leave low
	} 	

}

void MSound(u8 bPin, u8 cNotes, ...)
{
	va_list ap;
	word uDur;
	word uFreq;
	va_start(ap, cNotes);

	while (cNotes > 0) {
		uDur = va_arg(ap, unsigned int);
		uFreq = va_arg(ap, unsigned int);
		Sound(bPin, uDur, uFreq);
		cNotes--;
	}
	va_end(ap);
}

#ifdef __cplusplus
}
#endif
