//==============================================================================
// HSIO - Hardware serial port(s) IO. 
// 		some of definitions/functions from Minionbuilder
// 		work to integrate it with my own stuff and including interrupt based
//		serial port support.
//
// Version 0.1 - Work in Progress
//
// Major Contributers: Kurt Eckhardt 
// 		Robert V Mahoney Jr
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//==============================================================================
#ifndef _HSIO_H_
#define _HSIO_H_
#include "iopins.h"

#ifdef __cplusplus
extern "C" {
#endif

//-----------------------------------------------------------------------------
// Type definitions
//-----------------------------------------------------------------------------
typedef enum {
	HSERSTAT_CLEAR_INPUT_BUFFER,
	HSERSTAT_CLEAR_OUTPUT_BUFFER,
	HSERSTAT_CLEAR_BOTH_BUFFERS,
	HSERSTAT_INPUT_AVAILABLE,
	HSERSTAT_OUTPUT_EMPTY
} HSERSTAT_OPTIONS;	


extern void enablehserial (void);
extern void sethserial (u32 _lbaudrate, u8 _bdatabits, u8 _bparity, u8 _bstopbits);
extern void hserout (u8 *_psdata, u16 _wcount);
extern u16 hserin (u32 _ltimeout, u8 *_psdata, u16 _rcount, u16 _bterminator=0xffff);
extern bool hserstat(HSERSTAT_OPTIONS ho);  // ? - would it be better to simply have a function for each???
extern byte hseravail(void);	// How many characters are available?

extern void hsio_setbaud (u32 _lbaudrate);


#ifndef BASICATOMPRO28	// PRO40 and Arc32 have two serial ports...
extern void enablehserial2 (void);
extern void sethserial2 (u32 _lbaudrate, u8 _bdatabits, u8 _bparity, u8 _bstopbits);
extern void hserout2 (u8 *_psdata, u16 _wcount);
extern u16 hserin2 (u32 _ltimeout, u8 *_psdata, u16 _rcount, u16 _bterminator=0xffff);
extern bool hserstat2(HSERSTAT_OPTIONS ho);  // ? - would it be better to simply have a function for each???
extern byte hseravail2(void);	// How many characters are available?

void hsio_setbaud2 (u32 _lbaudrate);

#endif

#ifdef __cplusplus
}
#endif


#endif	/* _HSIO_H_ */
