//==============================================================================
// Servo.cpp - This file contains servo support that hopefully should be similiar
//		to the hservo system used in basic.  Will first start off just getting
//		servos to work, will then add in the timed group moves.
//  Not sure if unit base.  I am thinking in uSs as that is what most people
//	are used to...  Internally we will convert everything to clock tick units...
//
// Version 0.1 - Work in Progress
//
// Major Contributers: Kurt Eckhardt 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//==============================================================================

// >>>>>>>>>>>>>>>>>>>>>>> Warning needs work!!!!! <<<<<<<<<<<<<<<<<<<<<<<<<<<<

#include "baplib.h"
#include "Servo.h"

// For each servo we will store
// 
#define NUMSERVOS				20	// We are set up for a maximum of 20 servos
#define NUMTIMERCOMPAREMATCH	4	// Number of timer compare match units we have
#define NUMOVFPERCYCLE			5	// number of cycles of the timer per 20ms servo cycle
typedef struct _hsitem
{
	u16		wPW;			// Current pulse width in clocks
	u16		wPWEnd;			// Desired pulse width after a group move
	u16		wDeltaPW;		// how much to change per cycle
} HSITEM;

typedef struct _hsevero_data
{
	HSITEM	ahsi[NUMSERVOS];
	u16		wGroupMovePW[NUMSERVOS];	// place to remember new values for a group move.
	u8		cServosActive;		// Are there any servos active?
	u8 		bGroupMove;			// are we in a group move? - keep a counter so can nest...
	u8		iServo;				// which servo to start with for this pass
	u8		wFudge;				// fudge factor to get closer to exact pulses
	u8		bWhichIntsToProcess; // combination of which are enabled and which have happened.
	u8		abActivePins[4];	// know which pins are active. = 0xff if not active...
} HSERVO_DATA;

HSERVO_DATA g_hservoD;



#ifdef BASICATOMPRO28
#define clockCyclesPerMicrosecond 16								// We are running at 16mhz
#define usToTicks(_us)    (unsigned int)(((unsigned long) clockCyclesPerMicrosecond * _us) / 8)     // converts microseconds to tick (assumes prescale of 8)  // 12 Aug 2009
#else	// BAP40 and Arc32
#define clockCyclesPerMicrosecond 20								// We are running at 20mhz
#endif
//#define ticksToUs(_ticks) (( (unsigned long)_ticks * 8)/ clockCyclesPerMicrosecond ) // converts from ticks back to microseconds
#define usToTicks(_us)    (unsigned int)(((unsigned long) clockCyclesPerMicrosecond * _us) / 8)     // converts microseconds to tick (assumes prescale of 8)  
#define ticksToUs(_ticks) (( (unsigned long)_ticks * 8)/ (unsigned long)clockCyclesPerMicrosecond ) // converts from ticks back to microseconds



#define TRIM_DURATION       2                               // compensation ticks to trim adjust for digitalWrite delays // 12 August 2009


//=============================================================================
// Inline function that is called to update each servos position in handling
// a timed servo move
//=============================================================================
__inline__ void IncrementalUpdateServoPos(HSITEM* phsi) __attribute__ ((always_inline));
__inline__ void IncrementalUpdateServoPos(HSITEM* phsi)
{
	// Also see if we need to adjust the pulse width for the next time...
	if (phsi->wDeltaPW) {
		if (phsi->wPW < phsi->wPWEnd) {
			phsi->wPW += phsi->wDeltaPW;
			if (phsi->wPW >= phsi->wPWEnd){ // reached the end point
				phsi->wPW = phsi->wPWEnd;
				phsi->wDeltaPW = 0;
			}
		} else {
			phsi->wPW -= phsi->wDeltaPW;
			if (phsi->wPW <= phsi->wPWEnd){ // reached the end point
				phsi->wPW = phsi->wPWEnd;
				phsi->wDeltaPW = 0;
			}
		}
	}
}

#ifdef BASICATOMPRO28
//=============================================================================
// TIMERW() - Main timerw functino that is used to handle the background processing
// of the servo sub-system
//=============================================================================
extern "C" void TIMERW() __attribute__ ((interrupt_handler));
extern "C" void TIMERW()
{
	DBGPrintf("***TIMERW***\r");
	// See why we woke up for
	if (TW.TSRW.BIT.OVF) {
		// See if we need to set any of the pins back low now.
		if (g_hservoD.abActivePins[0] != 0xff)
			PI_LOW((*g_pPortTable)[g_hservoD.abActivePins[0]]);
		if (g_hservoD.abActivePins[1] != 0xff)
			PI_LOW((*g_pPortTable)[g_hservoD.abActivePins[1]]);
		if (g_hservoD.abActivePins[2] != 0xff)
			PI_LOW((*g_pPortTable)[g_hservoD.abActivePins[2]]);
		if (g_hservoD.abActivePins[3] != 0xff)
			PI_LOW((*g_pPortTable)[g_hservoD.abActivePins[3]]);

		// If no servos are currently active we can now bail as we have
		// set all of the previously active servos back to low...
		if (!g_hservoD.cServosActive) {
			TW.TSRW.BYTE = 0;		// turn off all of the status bytes.
			TW.TIERW.BYTE = 0;		// Turn off all interrupts
			return;					// and return...
		}

		// Clock over run.  we are ready to setup for the next pass.
		// setup TCNT such that we interrupt every 4mS.  For each 4ms we handle 1 servo per GRx value so
		// 4 servos.  So every 5 times through is one complete cycle which equals 20ms
		TW.TCNT = 1536-90;				// set such that 65536-1536=64000tics per interrupt/16MperHz = 4000uS= 4ms (fudge)
		g_hservoD.wFudge = 103;				// init the fudge factor...
		g_hservoD.iServo++; 
		if (g_hservoD.iServo >= NUMSERVOS) {
			g_hservoD.iServo = 0;
		}
			
		// Now lets start up to see if which servo we need to process in this pass
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);
			
			// servo is active so lets get all of the timers and the like setup
			TW.GRA = 65535 - g_hservoD.ahsi[g_hservoD.iServo].wPW - g_hservoD.wFudge;  // off by 1 but should work
			g_hservoD.wFudge -= 29;
			TW.TIERW.BIT.IMIEA = 1;		// enable that interrupt
			g_hservoD.abActivePins[0] = g_hservoD.iServo;	// remember which pin we are using
				
		} else 
			g_hservoD.abActivePins[0] = 0xff;	// remember which pin we are using

		// Channel B
		g_hservoD.iServo++;
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);

			// servo is active so lets get all of the timers and the like setup
			TW.GRB = 65535 - g_hservoD.ahsi[g_hservoD.iServo].wPW - g_hservoD.wFudge;  // off by 1 but should work
			g_hservoD.wFudge -= 29;
			TW.TIERW.BIT.IMIEB = 1;		// enable that interrupt
			g_hservoD.abActivePins[1] = g_hservoD.iServo;	// remember which pin we are using
		} else 
			g_hservoD.abActivePins[1] = 0xff;	// remember which pin we are using

		// Channel C
		g_hservoD.iServo++;
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);

			// servo is active so lets get all of the timers and the like setup
			TW.GRC = 65535 - g_hservoD.ahsi[g_hservoD.iServo].wPW - g_hservoD.wFudge;  // off by 1 but should work
			g_hservoD.wFudge += 29;
			TW.TIERW.BIT.IMIEC = 1;		// enable that interrupt
			g_hservoD.abActivePins[2] = g_hservoD.iServo;	// remember which pin we are using
		} else 
			g_hservoD.abActivePins[2] = 0xff;

		// Channel D
		g_hservoD.iServo++;
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);

			// servo is active so lets get all of the timers and the like setup
			TW.GRD = 65535 - g_hservoD.ahsi[g_hservoD.iServo].wPW - g_hservoD.wFudge;  // off by 1 but should work
			TW.TIERW.BIT.IMIED = 1;		// enable that interrupt
			g_hservoD.abActivePins[3] = g_hservoD.iServo;	// remember which pin we are using
		}
		else 
			g_hservoD.abActivePins[3] = 0xff;

		TW.TSRW.BYTE = 0;								// Clear all of the status bits
		return;											// and finally return
	}

	// Lets see which of the 4 channels we woke up for.  Lets try with pre-generated bitmask.
	g_hservoD.bWhichIntsToProcess = (TW.TSRW.BYTE & TW.TIERW.BYTE);
	if (g_hservoD.bWhichIntsToProcess & 0x01) {				// GRA Match
	//if (TW.TSRW.BIT.IMFA && TW.TIERW.BIT.IMIEA)	{		// GRA Match? and that interrupt is enabled...
		PI_HIGH((*g_pPortTable)[g_hservoD.abActivePins[0]]);	// set the servo pin high!
		TW.TSRW.BIT.IMFA = 0;	
		TW.TIERW.BIT.IMIEA = 0;								// clear the interrupt request as well
	}

	if (g_hservoD.bWhichIntsToProcess & 0x02) {				// GRB Match
	//if (TW.TSRW.BIT.IMFB && TW.TIERW.BIT.IMIEB)	{		// GRB Match?
		PI_HIGH((*g_pPortTable)[g_hservoD.abActivePins[1]]);	// Like A
		TW.TSRW.BIT.IMFB = 0;	
		TW.TIERW.BIT.IMIEB = 0;	
	}
	
	if (g_hservoD.bWhichIntsToProcess & 0x04) {				// GRC Match
	//if (TW.TSRW.BIT.IMFC && TW.TIERW.BIT.IMIEC) {		// GRC Match?
		PI_HIGH((*g_pPortTable)[g_hservoD.abActivePins[2]]);	// Like A
		TW.TSRW.BIT.IMFC = 0;	
		TW.TIERW.BIT.IMIEC = 0;	
	}

	if (g_hservoD.bWhichIntsToProcess & 0x08) {				// GRD Match
	//if (TW.TSRW.BIT.IMFD && TW.TIERW.BIT.IMIED)	{		// GRD Match?
		PI_HIGH((*g_pPortTable)[g_hservoD.abActivePins[3]]);	// Like A
		TW.TSRW.BIT.IMFD = 0;	
		TW.TIERW.BIT.IMIED = 0;	
	}
}
#else
//=============================================================================
// TIMERZ0() - Main timerw functino that is used to handle the background processing
// of the servo sub-system
//=============================================================================
extern "C" void TIMERZ0() __attribute__ ((interrupt_handler));
extern "C" void TIMERZ0()
{
	DBGPrintf("***TIMERW***\r");
	// See why we woke up for
	if (TZ0.TSR.BIT.OVF) {
		// See if we need to set any of the pins back low now.
		if (g_hservoD.abActivePins[0] != 0xff)
			PI_LOW((*g_pPortTable)[g_hservoD.abActivePins[0]]);
		if (g_hservoD.abActivePins[1] != 0xff)
			PI_LOW((*g_pPortTable)[g_hservoD.abActivePins[1]]);
		if (g_hservoD.abActivePins[2] != 0xff)
			PI_LOW((*g_pPortTable)[g_hservoD.abActivePins[2]]);
		if (g_hservoD.abActivePins[3] != 0xff)
			PI_LOW((*g_pPortTable)[g_hservoD.abActivePins[3]]);

		// If no servos are currently active we can now bail as we have
		// set all of the previously active servos back to low...
		if (!g_hservoD.cServosActive) {
			TZ0.TSR.BYTE = 0;		// turn off all of the status bytes.
			TZ0.TIER.BYTE = 0;		// Turn off all interrupts
			return;					// and return...
		}

		// Clock over run.  we are ready to setup for the next pass.
		// setup TCNT such that we interrupt every 4mS.  For each 4ms we handle 1 servo per GRx value so
		// 4 servos.  So every 5 times through is one complete cycle which equals 20ms
		TZ0.TCNT = 1536-90;				// set such that 65536-1536=64000tics per interrupt/16MperHz = 4000uS= 4ms (fudge)
		g_hservoD.wFudge = 103;				// init the fudge factor...
		g_hservoD.iServo++; 
		if (g_hservoD.iServo >= NUMSERVOS) {
			g_hservoD.iServo = 0;
		}
			
		// Now lets start up to see if which servo we need to process in this pass
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);
			
			// servo is active so lets get all of the timers and the like setup
			TZ0.GRA = 65535 - g_hservoD.ahsi[g_hservoD.iServo].wPW - g_hservoD.wFudge;  // off by 1 but should work
			g_hservoD.wFudge -= 29;
			TZ0.TIER.BIT.IMIEA = 1;		// enable that interrupt
			g_hservoD.abActivePins[0] = g_hservoD.iServo;	// remember which pin we are using
				
		} else 
			g_hservoD.abActivePins[0] = 0xff;	// remember which pin we are using

		// Channel B
		g_hservoD.iServo++;
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);

			// servo is active so lets get all of the timers and the like setup
			TZ0.GRB = 65535 - g_hservoD.ahsi[g_hservoD.iServo].wPW - g_hservoD.wFudge;  // off by 1 but should work
			g_hservoD.wFudge -= 29;
			TZ0.TIER.BIT.IMIEB = 1;		// enable that interrupt
			g_hservoD.abActivePins[1] = g_hservoD.iServo;	// remember which pin we are using
		} else 
			g_hservoD.abActivePins[1] = 0xff;	// remember which pin we are using

		// Channel C
		g_hservoD.iServo++;
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);

			// servo is active so lets get all of the timers and the like setup
			TZ0.GRC = 65535 - g_hservoD.ahsi[g_hservoD.iServo].wPW - g_hservoD.wFudge;  // off by 1 but should work
			g_hservoD.wFudge += 29;
			TZ0.TIER.BIT.IMIEC = 1;		// enable that interrupt
			g_hservoD.abActivePins[2] = g_hservoD.iServo;	// remember which pin we are using
		} else 
			g_hservoD.abActivePins[2] = 0xff;

		// Channel D
		g_hservoD.iServo++;
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);

			// servo is active so lets get all of the timers and the like setup
			TZ0.GRD = 65535 - g_hservoD.ahsi[g_hservoD.iServo].wPW - g_hservoD.wFudge;  // off by 1 but should work
			TZ0.TIER.BIT.IMIED = 1;		// enable that interrupt
			g_hservoD.abActivePins[3] = g_hservoD.iServo;	// remember which pin we are using
		}
		else 
			g_hservoD.abActivePins[3] = 0xff;

		TZ0.TSR.BYTE = 0;								// Clear all of the status bits
		return;											// and finally return
	}

	// Lets see which of the 4 channels we woke up for.  Lets try with pre-generated bitmask.
	g_hservoD.bWhichIntsToProcess = (TZ0.TSR.BYTE & TZ0.TIER.BYTE);
	if (g_hservoD.bWhichIntsToProcess & 0x01) {				// GRA Match
	//if (TZ0.TSR.BIT.IMFA && TZ0.TIER.BIT.IMIEA)	{		// GRA Match? and that interrupt is enabled...
		PI_HIGH((*g_pPortTable)[g_hservoD.abActivePins[0]]);	// set the servo pin high!
		TZ0.TSR.BIT.IMFA = 0;	
		TZ0.TIER.BIT.IMIEA = 0;								// clear the interrupt request as well
	}

	if (g_hservoD.bWhichIntsToProcess & 0x02) {				// GRB Match
	//if (TZ0.TSR.BIT.IMFB && TZ0.TIER.BIT.IMIEB)	{		// GRB Match?
		PI_HIGH((*g_pPortTable)[g_hservoD.abActivePins[1]]);	// Like A
		TZ0.TSR.BIT.IMFB = 0;	
		TZ0.TIER.BIT.IMIEB = 0;	
	}
	
	if (g_hservoD.bWhichIntsToProcess & 0x04) {				// GRC Match
	//if (TZ0.TSR.BIT.IMFC && TZ0.TIER.BIT.IMIEC) {		// GRC Match?
		PI_HIGH((*g_pPortTable)[g_hservoD.abActivePins[2]]);	// Like A
		TZ0.TSR.BIT.IMFC = 0;	
		TZ0.TIER.BIT.IMIEC = 0;	
	}

	if (g_hservoD.bWhichIntsToProcess & 0x08) {				// GRD Match
	//if (TZ0.TSR.BIT.IMFD && TZ0.TIER.BIT.IMIED)	{		// GRD Match?
		PI_HIGH((*g_pPortTable)[g_hservoD.abActivePins[3]]);	// Like A
		TZ0.TSR.BIT.IMFD = 0;	
		TZ0.TIER.BIT.IMIED = 0;	
	}
}


#endif


//==============================================================================
// Init the ServoGroupMove data.
//==============================================================================
void HServoInit(void)			// init our stuff
{
	DBGPrintf("HServoInit\r");
#ifdef BASICATOMPRO28
	TW.TCNT = 0;				// give us some delay before the first interrupt...
	TW.TMRW.BYTE = 0x80;		// CTS clock no PWMD...
	TW.TSRW.BYTE = 0;			// clear out all status bytes
	TW.TIOR0.BIT.IOA = 0;		// output match - don't change any io pins
	TW.TIOR0.BIT.IOB = 0;		// output match - don't change any io pins
	TW.TIOR1.BIT.IOC = 0;		// output match - don't change any io pins
	TW.TIOR1.BIT.IOD = 0;		// output match - don't change any io pins
	
	if (TW.TIERW.BIT.OVIE == 0) {
		g_hservoD.abActivePins[0] = 0xff;
		g_hservoD.abActivePins[1] = 0xff;
		g_hservoD.abActivePins[2] = 0xff;
		g_hservoD.abActivePins[3] = 0xff;
		g_hservoD.iServo = 0xff; 		// start off with a high value to start off at 0
	}

	TW.TIERW.BIT.OVIE = 1;		// Interrupt on overflow

#else //BAP40 and arc32
	TZ0.TCNT = 0;				// give us some delay before the first interrupt...
	TZ.TSTR.BIT.STR0 = 1;		// Make sure Timer Z0 is running 
	TZ0.TSR.BYTE = 0;			// clear out all status bytes
	TZ0.TIORA.BIT.IOA = 0;		// output match - don't change any io pins
	TZ0.TIORA.BIT.IOB = 0;		// output match - don't change any io pins
	TZ0.TIORC.BIT.IOC = 0;		// output match - don't change any io pins
	TZ0.TIORC.BIT.IOD = 0;		// output match - don't change any io pins
	
	if (TZ0.TIER.BIT.OVIE == 0) {
		g_hservoD.abActivePins[0] = 0xff;
		g_hservoD.abActivePins[1] = 0xff;
		g_hservoD.abActivePins[2] = 0xff;
		g_hservoD.abActivePins[3] = 0xff;
		g_hservoD.iServo = 0xff; 		// start off with a high value to start off at 0
	}

	TZ0.TIER.BIT.OVIE = 1;		// Interrupt on overflow

#endif
			
	enable();					// Make sure interrupts are enabled...

}

//==============================================================================
// HServo - start up a group move...
//==============================================================================
void HServoGroupMoveStart(void)						// start a group move
{
	u8 i;

	DBGPrintf("HServoGroupMoveStart\r");
	
	// Start up a group move.  If we are not already in a group move
	// initialize the group move code so we can detect which servos
	// we will process...
	if (g_hservoD.bGroupMove++ == 0) {
		for (i=0; i < NUMSERVOS; i++)
			g_hservoD.wGroupMovePW[i] = 0xffff;		// set to value we can detect
	}			
}

//==============================================================================
// HServoGroupMoveCommit - Commit any group move up a group that is in process
//==============================================================================
void HServoGroupMoveCommit(u16 wMoveTime)			// Do the commit, pass in how long you wish for the move to take
{
	u8 		i;
	DBGPrintf("HServoGroupMoveCommit %d\r", wMoveTime);
	if (wMoveTime)
		wMoveTime = (wMoveTime + 10)/20;			// convert to number of servo cycles.
	// Now do the actual group move... Detect count out of whack...
	if (g_hservoD.bGroupMove) {
		if (--g_hservoD.bGroupMove == 0) {
			// Ok it is time to commit all of the values...
			for (i=0; i < NUMSERVOS; i++) {
				if (g_hservoD.wGroupMovePW[i] != 0xffff) {
					// need to keep track of how many servos we have active 
					if (g_hservoD.wGroupMovePW[i] && !g_hservoD.ahsi[i].wPW) {
						if (++g_hservoD.cServosActive == 1)
							HServoInit();
					} else if (!g_hservoD.wGroupMovePW[i] && g_hservoD.ahsi[i].wPW) {
						g_hservoD.cServosActive--;
					}
					g_hservoD.ahsi[i].wDeltaPW = 0;								// Assume no delta, set last to minimize race conditions.
					if ((wMoveTime == 0) || (g_hservoD.ahsi[i].wPW == 0)) {
						g_hservoD.ahsi[i].wPW = g_hservoD.wGroupMovePW[i];		// set the PW to the new value
						g_hservoD.ahsi[i].wPWEnd =  g_hservoD.wGroupMovePW[i];	// Set the desired to the same
					} else {
						// First if we have not already made it to a previous timed
						g_hservoD.ahsi[i].wPW =  g_hservoD.ahsi[i].wPWEnd; 		// move location, get there now...
						g_hservoD.ahsi[i].wPWEnd =  g_hservoD.wGroupMovePW[i];	// Set the desired to the new time
						
						// calculate the delta per cycle..
						g_hservoD.ahsi[i].wDeltaPW = (iabs(g_hservoD.ahsi[i].wPWEnd - 
								g_hservoD.ahsi[i].wPW) + wMoveTime/2) / wMoveTime;
					}					
				}
			}
		}
	}	
#if 0	
	for (i=0; i < NUMSERVOS; i++) {
		DBGPrintf("%d - %x %x %x\r", i, g_hservoD.ahsi[i].wPW, g_hservoD.ahsi[i].wPWEnd, g_hservoD.ahsi[i].wDeltaPW);
	}
#endif	
}


//==============================================================================
// HServo - start up a group move...
//==============================================================================
void HServo(u8 pin, u16 wPulseWidth)
{
	DBGPrintf("HServo %d %d\r", pin, wPulseWidth);
	// First convert the pulse width to clocks for the processor...  BUGBUG: first pass only BAP28...
	wPulseWidth *= 16;			// Clocks per millisecond.
	
	output(pin);	
	// See if we are in a group move...
	if (g_hservoD.bGroupMove) {
		g_hservoD.wGroupMovePW[pin] = wPulseWidth;
	} else {
		// need to keep track of how many servos we have active 
		if (wPulseWidth && !g_hservoD.ahsi[pin].wPW) {
			if (++g_hservoD.cServosActive == 1)
				HServoInit();
		} else if (!wPulseWidth && g_hservoD.ahsi[pin].wPW) {
			g_hservoD.cServosActive--;
		}
		
		g_hservoD.ahsi[pin].wDeltaPW = 0;			// No delta time stuff
		g_hservoD.ahsi[pin].wPW = wPulseWidth;		// set the PW to the new value
		g_hservoD.ahsi[pin].wPWEnd =  wPulseWidth;	// Set the desired to the same
	}
}


//==============================================================================
// Init the HServoIdle data.
//==============================================================================
bool HServoIdle(u8 pin)
{
	return (g_hservoD.ahsi[pin].wPW == g_hservoD.ahsi[pin].wPWEnd);
}


//==============================================================================
// Init the HServoPos data.
//==============================================================================
extern u16 HServoPos(u8 pin)
{
	return g_hservoD.ahsi[pin].wPW / 16;
}

